package com.community.huiju.common.hk;

import com.alibaba.fastjson.JSON;
import com.community.commom.hk.HttpClientSSLUtils;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import net.coobird.thumbnailator.Thumbnails;
import org.springframework.web.multipart.MultipartFile;
import java.io.*;
import com.community.commom.hk.Digests;

import java.util.Map;

/**
 * 海康 API
 * @author weiximei
 */
@Slf4j
public class HKOpenApi {

    public static String getMonitoryPoint(Map<String,Object> map){
        Map<String,Object> parMap = Maps.newHashMap();
        String url = HKConstant.OPENAPI_IP_PORT_HTTP + HKConstant.ITF_MONITORY_POINT;
        //设置APPKEY
        parMap.put("appkey", HKConstant.APPKEY);
        //设置时间参数
        parMap.put("time", System.currentTimeMillis());
        parMap.put("pageNo",map.get("pageNo"));
        parMap.put("pageSize",map.get("pageSize"));
        parMap.put("opUserUuid",map.get("opUserUuid") == null?HKConstant.OP_USER_UUID:map.get("opUserUuid"));
        parMap.put("unitUuids",map.get("unitUuids"));
        String params =  JSON.toJSONString(parMap);
        log.info("获取 监控列表 请求参数：{}", params);
        String data = null;
        try {
            data = HttpClientSSLUtils.doPost(url + "?token=" + Digests.buildToken(url + "?" + params, params, HKConstant.SECRET), params);
            log.info("获取 监控列表 请求返回结果：{}",data);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("获取监控列表失败!",e);
        }

        return data;
    }

    /**
     *
     * @param img_path 获取图片的扩展名
     * @return
     */
    public static String getPicSuffix(String img_path){
        if (img_path == null || img_path.indexOf(".") == -1){
            //如果图片地址为null或者地址中没有"."就返回""
            return "";
        }
        return img_path.substring(img_path.lastIndexOf(".") + 1).
                trim().toLowerCase();
    }
    
    /**
     * 将本地图片进行Base64位编码
     */
    public static String getImgStr(MultipartFile imgFile){
        //将图片文件转化为字节数组字符串，并对其进行Base64编码处理
        byte[] data = null;
        //读取图片字节数组
        try
        {
            data = imgFile.getBytes();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        return new String(org.apache.commons.codec.binary.Base64.encodeBase64(data));
    }
    
    /**
     * 将本地图片进行Base64位编码
     * @param fileBytes
     * @return
     */
    public static String getImgStr(byte [] fileBytes) {
        return new String(org.apache.commons.codec.binary.Base64.encodeBase64(fileBytes));
    }


    /**
     * 根据部门名称分页获取部门
     *      第几页, 一页多少行, 操作用户uuid
     * @param map
     * @return
     */
    public static String getDeptName(Map<String,Object> map){
        Map<String,Object> parMap = Maps.newHashMap();
        String url = HKConstant.OPENAPI_IP_PORT_HTTP + HKConstant.ITF_LIKE_DEPT_NAME;
        //设置APPKEY
        parMap.put("appkey", HKConstant.APPKEY);
        //设置时间参数
        parMap.put("time", System.currentTimeMillis());
        parMap.put("pageNo",map.get("pageNo"));
        parMap.put("pageSize",map.get("pageSize"));
        parMap.put("opUserUuid",map.get("opUserUuid") == null?HKConstant.OP_USER_UUID:map.get("opUserUuid"));
        parMap.put("deptName",map.get("deptName"));
        String params =  JSON.toJSONString(parMap);
        log.info("根据部门名称分页获取部门 请求参数：{}", params);
        String data = null;
        try {
            data = HttpClientSSLUtils.doPost(url + "?token=" + Digests.buildToken(url + "?" + params, params, HKConstant.SECRET), params);
            log.info("根据部门名称分页获取部门 请求返回结果：{}",data);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("根据部门名称分页获取部门失败!",e);
        }

        return data;
    }


    /**
     * 根据指定大小压缩图片
     *
     * @param imageBytes  源图片字节数组
     * @param desFileSize 指定图片大小，单位kb
     * @param imageId     影像编号
     * @return 压缩质量后的图片字节数组
     */
    public static byte[] compressPicForScale(byte[] imageBytes, long desFileSize, String imageId) {
        if (imageBytes == null || imageBytes.length <= 0 || imageBytes.length < desFileSize * 1024) {
            return imageBytes;
        }
        long srcSize = imageBytes.length;
        double accuracy = getAccuracy(srcSize / 1024);
        try {
            while (imageBytes.length > desFileSize * 1024) {
                ByteArrayInputStream inputStream = new ByteArrayInputStream(imageBytes);
                ByteArrayOutputStream outputStream = new ByteArrayOutputStream(imageBytes.length);
                Thumbnails.of(inputStream)
                        .scale(accuracy)
                        .outputQuality(accuracy)
                        .toOutputStream(outputStream);
                imageBytes = outputStream.toByteArray();
            }
            log.info("【图片压缩】imageId={} | 图片原大小={}kb | 压缩后大小={}kb",
                    imageId, srcSize / 1024, imageBytes.length / 1024);
        } catch (Exception e) {
            log.error("【图片压缩】msg=图片压缩失败!", e);
        }
        return imageBytes;
    }

    /**
     * 自动调节精度(经验数值)
     *
     * @param size 源图片大小
     * @return 图片压缩质量比
     */
    private static double getAccuracy(long size) {
        double accuracy;
        if (size < 900) {
            accuracy = 0.85;
        } else if (size < 2047) {
            accuracy = 0.6;
        } else if (size < 3275) {
            accuracy = 0.44;
        } else {
            accuracy = 0.4;
        }
        return accuracy;
    }

    /**
     *
     * 添加人员
     *
     *      比如 住户
     */
    public static String getAddUser(Map<String,Object> map){
        Map<String,Object> parMap = Maps.newHashMap();
        String url = HKConstant.OPENAPI_IP_PORT_HTTP + HKConstant.ITF_LIKE_DEPT_NAME;
        //设置APPKEY
        parMap.put("appkey", HKConstant.APPKEY);
        //设置时间参数
        parMap.put("time", System.currentTimeMillis());
        parMap.put("pageNo",map.get("pageNo"));
        parMap.put("pageSize",map.get("pageSize"));
        parMap.put("opUserUuid",map.get("opUserUuid") == null?HKConstant.OP_USER_UUID:map.get("opUserUuid"));
        parMap.put("personNo",map.get("personNo"));
        parMap.put("personName",map.get("personName"));
        String params =  JSON.toJSONString(parMap);
        log.info("添加人员 请求参数：{}", params);
        String data = null;
        try {
            data = HttpClientSSLUtils.doPost(url + "?token=" + Digests.buildToken(url + "?" + params, params, HKConstant.SECRET), params);
            log.info("添加人员 请求返回结果：{}",data);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("添加人员失败!",e);
        }

        return data;
    }
    
    
    /**
     * 根据中心UUID集获取中心下的门禁设备信息
     *
     * @return
     */
    public static String getACDevicesByUnitUuids(){
        Map<String,Object> parMap = Maps.newHashMap();
        String url = HKConstant.OPENAPI_IP_PORT_HTTP + HKConstant.ITF_MONITORY_POINT;
        //设置APPKEY
        parMap.put("appkey", HKConstant.APPKEY);
        //设置时间参数
        parMap.put("time", System.currentTimeMillis());
        parMap.put("pageNo",1);
        parMap.put("pageSize",10);
        parMap.put("opUserUuid",HKConstant.OP_USER_UUID);
        parMap.put("unitUuids","1048576");
        String params =  JSON.toJSONString(parMap);
        log.info("获取中心UUID集获取中心下的门禁设备请求参数：{}", params);
        String data = null;
        try {
            data = HttpClientSSLUtils.doPost(url + "?token=" + Digests.buildToken(url + "?" + params, params, HKConstant.SECRET), params);
            log.info("获取 中心UUID集获取中心下的门禁设备 请求返回结果：{}",data);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("获取失败!",e);
        }
        
        return data;
    }
    
    /**
     * 分页获取门禁点信息。
     *
     * @return
     */
    public static String getDoors(){
        Map<String,Object> parMap = Maps.newHashMap();
        String url = HKConstant.OPENAPI_IP_PORT_HTTP + HKConstant.ITF_MONITORY_POINT;
        //设置APPKEY
        parMap.put("appkey", HKConstant.APPKEY);
        //设置时间参数
        parMap.put("time", System.currentTimeMillis());
        parMap.put("pageNo",1);
        parMap.put("pageSize",10);
        parMap.put("opUserUuid",HKConstant.OP_USER_UUID);
        String params =  JSON.toJSONString(parMap);
        log.info("分页获取门禁点信息请求参数：{}", params);
        String data = null;
        try {
            data = HttpClientSSLUtils.doPost(url + "?token=" + Digests.buildToken(url + "?" + params, params, HKConstant.SECRET), params);
            log.info("分页获取门禁点信息 请求返回结果：{}",data);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("获取失败!",e);
        }
        
        return data;
    }
    
    /**
     * 批量获取控制中心信息，多个中心的UUID用“,”分割组成的字符串。
     *
     * @return
     */
    public static String getUnitsByUuids(){
        Map<String,Object> parMap = Maps.newHashMap();
        String url = HKConstant.OPENAPI_IP_PORT_HTTP + HKConstant.ITF_MONITORY_POINT;
        //设置APPKEY
        parMap.put("appkey", HKConstant.APPKEY);
        //设置时间参数
        parMap.put("time", System.currentTimeMillis());
        parMap.put("pageNo",1);
        parMap.put("pageSize",10);
        parMap.put("opUserUuid",HKConstant.OP_USER_UUID);
        parMap.put("unitUuids","1048576");
        String params =  JSON.toJSONString(parMap);
        log.info("获取控制中心信息请求参数：{}", params);
        String data = null;
        try {
            data = HttpClientSSLUtils.doPost(url + "?token=" + Digests.buildToken(url + "?" + params, params, HKConstant.SECRET), params);
            log.info("获取控制中心信息 请求返回结果：{}",data);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("获取失败!",e);
        }
        
        return data;
    }
    
    /**
     *  根据中心UUID分页获取下级区域
     * @return
     */
    public static String getRegionsByUnitUuid(){
        Map<String,Object> parMap = Maps.newHashMap();
        String url = HKConstant.OPENAPI_IP_PORT_HTTP + HKConstant.ITF_MONITORY_POINT;
        //设置APPKEY
        parMap.put("appkey", HKConstant.APPKEY);
        //设置时间参数
        parMap.put("time", System.currentTimeMillis());
        parMap.put("pageNo",1);
        parMap.put("pageSize",10);
        parMap.put("opUserUuid",HKConstant.OP_USER_UUID);
        parMap.put("parentUuid","1048576");
        parMap.put("allChildren",1);
        String params =  JSON.toJSONString(parMap);
        log.info("根据中心UUID分页获取下级区域请求参数：{}", params);
        String data = null;
        try {
            data = HttpClientSSLUtils.doPost(url + "?token=" + Digests.buildToken(url + "?" + params, params, HKConstant.SECRET), params);
            log.info("根据中心UUID分页获取下级区域 请求返回结果：{}",data);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("获取失败!",e);
        }
        
        return data;
    }
    
    /**
     * 根据区域UUID集获取门禁点
     * @return
     */
    public static String getDoorsByRegionUuids(){
        Map<String,Object> parMap = Maps.newHashMap();
        String url = HKConstant.OPENAPI_IP_PORT_HTTP + HKConstant.DOORS_BY_REGION_UUIDS;
        //设置APPKEY
        parMap.put("appkey", HKConstant.APPKEY);
        //设置时间参数
        parMap.put("time", System.currentTimeMillis());
        parMap.put("opUserUuid",HKConstant.OP_USER_UUID);
        parMap.put("regionUuids","2a36878c9a4a49d290d253ac1b4746dc");
        String params =  JSON.toJSONString(parMap);
        log.info("根据区域UUID集获取门禁点请求参数：{}", params);
        String data = null;
        try {
            data = HttpClientSSLUtils.doPost(url + "?token=" + Digests.buildToken(url + "?" + params, params, HKConstant.SECRET), params);
            log.info("根据区域UUID集获取门禁点 请求返回结果：{}",data);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("获取失败!",e);
        }
        
        return data;
    }
    
    /**
     * 根据人员ID集添加权限
     * @return
     */
    public static String addAuthoritiesByPersonIds(String personIds){
        Map<String,Object> parMap = Maps.newHashMap();
        String url = HKConstant.OPENAPI_IP_PORT_HTTP + HKConstant.ADD_AUTHORITIES_BY_PERSONIDS;
        //设置APPKEY
        parMap.put("appkey", HKConstant.APPKEY);
        //设置时间参数
        parMap.put("time", System.currentTimeMillis());
        parMap.put("opUserUuid",HKConstant.OP_USER_UUID);
        parMap.put("personIds",personIds);
        parMap.put("doorGroupUuids","");
        parMap.put("doorUuids","57231a05fd3b45f7a2570a28a85821c4,efcb9a3fe4e64e2f9c0f546c54465524,9da7c0073138453fa7782b629743cff7,85c22fde220d4b35ac13f805f05fd04c");
        String params =  JSON.toJSONString(parMap);
        log.info("根据人员ID集添加权限请求参数：{}", params);
        String data = null;
        try {
            data = HttpClientSSLUtils.doPost(url + "?token=" + Digests.buildToken(url + "?" + params, params, HKConstant.SECRET), params);
            log.info("根据人员ID集添加权限 请求返回结果：{}",data);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("获取失败!",e);
        }
        
        return data;
    }
    
    /**
     * 根据设备UUID集异动下载门禁权限
     * @return
     */
    public static String downloadAuthorityByDeviceUuids(){
        Map<String,Object> parMap = Maps.newHashMap();
        String url = HKConstant.OPENAPI_IP_PORT_HTTP + HKConstant.DOWNLOAD_AUTHORITY_BY_DEVICEUUIDS;
        //设置APPKEY
        parMap.put("appkey", HKConstant.APPKEY);
        //设置时间参数
        parMap.put("time", System.currentTimeMillis());
        parMap.put("opUserUuid",HKConstant.OP_USER_UUID);
        parMap.put("deviceUuids","57231a05fd3b45f7a2570a28a85821c4,efcb9a3fe4e64e2f9c0f546c54465524,9da7c0073138453fa7782b629743cff7,85c22fde220d4b35ac13f805f05fd04c");
        String params =  JSON.toJSONString(parMap);
        log.info("根据设备UUID集异动下载门禁权限请求参数：{}", params);
        String data = null;
        try {
            data = HttpClientSSLUtils.doPost(url + "?token=" + Digests.buildToken(url + "?" + params, params, HKConstant.SECRET), params);
            log.info("根据设备UUID集异动下载门禁权限 请求返回结果：{}",data);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("获取失败!",e);
        }
        
        return data;
    }
    
    /**
     * 指定人员人脸/指纹权限下载
     * @return
     */
    public static String downloadFaceAndFingerInfos(String personIds){
        Map<String,Object> parMap = Maps.newHashMap();
        String url = HKConstant.OPENAPI_IP_PORT_HTTP + HKConstant.DOWNLOAD_FACE_AND_FINGERINFOS;
        //设置APPKEY
        parMap.put("appkey", HKConstant.APPKEY);
        //设置时间参数
        parMap.put("time", System.currentTimeMillis());
        parMap.put("opUserUuid",HKConstant.OP_USER_UUID);
        parMap.put("deviceUuids","83dfab0eef52411d991d6e621296b60a,dfd80b22f00545b8b36fa981019df83c");
        parMap.put("personIds",personIds);
        parMap.put("type",1);
        String params =  JSON.toJSONString(parMap);
        log.info("指定人员人脸/指纹权限下载 请求参数：{}", params);
        String data = null;
        try {
            data = HttpClientSSLUtils.doPost(url + "?token=" + Digests.buildToken(url + "?" + params, params, HKConstant.SECRET), params);
            log.info("指定人员人脸/指纹权限下载 请求返回结果：{}",data);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("获取失败!",e);
        }
        
        return data;
    }


    public static void main(String[] args) {
        Map<String,Object> parMap = Maps.newHashMap();
        parMap.put("pageNo",1);
        parMap.put("pageSize",100);
        parMap.put("opUserUuid",HKConstant.OP_USER_UUID);
        parMap.put("deptName","住户");
        String result = HKOpenApi.getDeptName(parMap);
        System.out.println(result);
    }
}
